/* =================================================================
**
** File					: d3gauge.v2.js
** Version 				: 9.20
** Last update 		 	: 12/01/2023
**
** Copyright (c) 2020 by Need4Viz,
** All rights reserved.
**
** This software is the confidential and proprietary information
** of Need4Viz. You shall not disclose such Confidential
** Information and shall use it only in accordance with the terms
** of the @license agreement you entered into with Need4Viz.
**   
** ================================================================= */

'use strict';

class Gauge {
  constructor(configuration) {
    // default configuration settings
    const config = {
      size : 200,
	  arcPadding : 15,
	  arcWidth : 20,
	  arcCornerRound : 0,
	  pointerWidth : 6,
	  pointerTailLength : 5,
	  pointerTail : 'angular',
	  labelInset : 10,
	  minAngle : -90,
	  maxAngle : 90,
      margin : 10,
      minValue : 0,
      maxValue : 10,
      majorTicks : 5,
      lowThreshhold : 3,
      highThreshhold : 7,
      scale: 'linear',
	  ticksLength: 'large',
      lowThreshholdColor : '#009900',
      defaultColor : '#ffe500',
      highThreshholdColor : '#cc0000',
	  gradientColors : false,
      transitionMs : 1000,
      displayUnit: 'Value',
	  leftFootVal: '0 a 99%',
	  rightFootVal: '99 a 100%',
	  roundMes: 1,
	  unitMesPre: '',
	  unitMesSuf: '',
	  localeMes : 'fr-FR'	  
    };

    this.config = Object.assign(config, configuration);

	/** Pointer  **/
    const pointerHeadLength = this._radius()*.8 - 2*this.config.labelInset;
    this.lineData = [
        [this.config.pointerWidth / 2, 0],
        [0, -pointerHeadLength],
        [-(this.config.pointerWidth / 2), 0],
        [0, this.config.pointerTailLength],
        [this.config.pointerWidth / 2, 0]
      ];
	if(this.config.pointerTail == 'chevron'){
	 this.lineData = [
        [this.config.pointerWidth / 2, 0],
        [0, -pointerHeadLength],
        [-(this.config.pointerWidth / 2), 0],
        [0, -this.config.pointerTailLength],
        [this.config.pointerWidth / 2, 0]
      ];
	}
	this.roundData = 'M '+(this.config.pointerWidth / 2)+' 0 L 0 '+(-pointerHeadLength)+' L '+(-(this.config.pointerWidth / 2))+' 0 A '+(this.config.pointerWidth / 2)+' '+(this.config.pointerWidth / 2)+' 0 0 0 '+(this.config.pointerWidth / 2)+' 0';
	

	/** Angle for Ticks, Text number & Pointer **/
	this.angleRange = this.config.maxAngle - this.config.minAngle;
	if (config.scale == 'log') {
	  this.scale = d3.scaleLog()
		.range([0,1])
		.domain([config.minValue, config.maxValue]);
	}
	else {
	  this.scale = d3.scaleLinear()
	  .range([0,1])
	  .domain([config.minValue, config.maxValue]);
	}

	/** Color gradient */
	var pi = Math.PI;
	var dom_min = this.config.minAngle*pi/180;
	var dom_lowthr = dom_min + (this.config.lowThreshhold-this.config.minValue)/(this.config.maxValue-this.config.minValue)*(this.angleRange*pi/180);
	var dom_higthr = dom_min + (this.config.highThreshhold-this.config.minValue)/(this.config.maxValue-this.config.minValue)*(this.angleRange*pi/180);
	var dom_max = this.config.maxAngle*pi/180;
	/** Color normal **/
	const colorDomain = [config.lowThreshhold, config.highThreshhold].map(this.scale);
    const colorRange  = [
      config.lowThreshholdColor,
      config.defaultColor,
      config.highThreshholdColor
    ];
	if(this.config.gradientColors){
	  this.colorScale = d3.scaleLinear()
		//.domain([-pi/2, -pi/3,pi/10,pi/2])
		.domain([dom_min,dom_lowthr,dom_higthr,dom_max])
		.range([this.config.lowThreshholdColor, this.config.defaultColor, this.config.defaultColor,this.config.highThreshholdColor]);

	  this.colorScale2 = d3.scaleThreshold()
		.domain([dom_lowthr,dom_higthr])
		.range([this.config.lowThreshholdColor, this.config.defaultColor, this.config.highThreshholdColor]);
	} else {
	  this.colorScale = d3.scaleThreshold().domain(colorDomain).range(colorRange);
	}
  
	
	/** Ticks */	
    let ticks = config.majorTicks;
    if (config.scale === 'log') {
      ticks = Math.log10(config.maxValue/minValue);
    }
	this.ticks = this.scale.ticks(ticks);
		this.threshholds = [
		  config.minValue,
		  config.lowThreshhold,
		  config.highThreshhold,
		  config.maxValue
		]
		.map(d => this.scale(d));

	/** Arcs & Arrows : defining each arcs & arrows  : NEW */
	if(this.config.gradientColors){
		var prevAngle = dom_min;
		this.breakPoints = 500; // number of arcs & arrows
		this.lines = []  
		this.angleArr = [];
		this.arcArr = [];
		this.aScale = d3.scaleLinear()
		//  .range([-Math.PI/2,Math.PI/2])
		  .range([dom_min,dom_max])
		  .domain([0,this.breakPoints - 1]);
		this.innerRad = this._radius() - this.config.arcWidth - this.config.arcPadding;
		this.outerRad = this._radius()  - this.config.arcPadding;

		for(let i = 0; i < this.breakPoints; i++) {
		  this.angleArr[i] = this.aScale(i);
		  var singleArrow = [{"x":(this.innerRad*Math.sin(this.angleArr[i])), "y":-(this.innerRad*Math.cos(this.angleArr[i]))},{ "y":-(this.outerRad*Math.cos(this.angleArr[i])), "x":(this.outerRad*Math.sin(this.angleArr[i]))}];
		  var subArc = {};
		  this.lines.push(singleArrow);
		  subArc["start"] = prevAngle; 
		  subArc["end"] = this.angleArr[i];
		  prevAngle = this.angleArr[i];   
		  this.arcArr.push(subArc);   
		};
	} else {
		this.arc = d3.arc()
		  .innerRadius(this._radius() - this.config.arcWidth - this.config.arcPadding)
		  .outerRadius(this._radius() - this.config.arcPadding)
		  .cornerRadius(this.config.arcCornerRound)
		  .startAngle((d, i) => {
			const ratio = i > 0 ? this.threshholds[i-1] : this.threshholds[0];
			return this._deg2rad(this.config.minAngle + (ratio * this.angleRange));
		  })
		  .endAngle((d, i) => this._deg2rad(this.config.minAngle + this.threshholds[i] * this.angleRange));
	}
  }

	/** Method */
	_radius() {
	return (this.config.size - this.config.margin) / 2;
	}

	_deg2rad(deg) {
	return deg * Math.PI / 180;
	}

	/** Draw chart */
	render(container, newValue) {

	if(this.config.gradientColors){
		//Inside arcs : this => path.arc
		var aArcArr = this.arcArr;
		var aInnerRad = this.innerRad;
		var aOuterRad = this.outerRad;
		var aAngleArr = this.angleArr;
		var aColorScale = this.colorScale;

		//For arrows
		var line = d3.line()
		  .x(function (d) { return d.x; })
		  .y(function (d) { return d.y; }); //for arrow
		var aLines = this.lines;
	}
	
	const svg = d3.select(container)
		.append('svg')
		.attr('class', 'gauge')
		.attr('width', this.config.size + this.config.margin)
		.attr('height', this.config.size / 2 + this.config.margin);

	// display panel arcs with color scale
	const arcs = svg.append('g')
		.attr('class', 'arc')
		.attr('transform', `translate(${this._radius()}, ${this._radius()})`);


	// draw the color arcs
	if(this.config.gradientColors){
	  arcs.selectAll('arcs')
		.data(aArcArr)
		.enter()
		.append('path').attr("class","arc")
		.attr('d', function(d,i) {
		  return d3.arc()
		  .innerRadius(aInnerRad)
		  .outerRadius(aOuterRad)
		   .startAngle(d.end)
		   .endAngle(d.start)()}).attr("fill",function(d,i) { 
		 return aColorScale(aAngleArr[i])})
		 
	  // draw the color arrows
	  svg.append('g').attr('class','arrow').attr('transform', `translate(${this._radius()}, ${this._radius()})`)
	   .selectAll("line")
	   .data(aLines)
	   .enter()
	   .append("path").attr("class","arrow").attr("d", line).attr("stroke",function(d,i) { 
		return aColorScale(aAngleArr[i])})
	} else {
	  arcs.selectAll('path')
		.data(this.threshholds)
		.enter()
		.append('path')
		.attr('fill', d => this.colorScale(d-0.001))
		.attr('class', d => {if (d>0) return 'stroked_arc'})
		.attr('d', this.arc);
	}
	
	  // display panel - labels
	  const lg = svg.append('g')
		.attr('class', 'label')
		.attr('transform', `translate(${this._radius()},${this._radius()})`);

	  // display panel - text
	  lg.selectAll('text')
		.data(this.ticks)
		.enter()
		.append('text')
		.attr('transform', d => {
		   var newAngle = this.config.minAngle + (this.scale(d) * this.angleRange);
		   return `rotate(${newAngle}) translate(0, ${this.config.labelInset - this._radius()})`;
		 })
		.text(d3.format('1,.0f'));

	  // display panel - ticks
	  lg.selectAll('line')
		.data(this.ticks)
		.enter()
		.append('line')
		.attr('class', 'tickline')
		.attr('x1', 0)
		.attr('y1', 20-this.config.arcWidth)
		.attr('x2', 0);
	   
	   if(this.config.ticksLength == 'large'){
		lg.selectAll('line')
		  .attr('y2', 30);
	   }
	   else{
		lg.selectAll('line')
		  .attr('y2', 30-this.config.arcWidth);
	   }
	   
	   lg.selectAll('line')
		 .attr('transform', d => {
		  const newAngle = this.config.minAngle + (this.scale(d) * this.angleRange);
		  return `rotate(${newAngle}), translate(0, ${this.config.arcWidth  - this.config.labelInset - this._radius()})`;
		});

	  // display pointer
	  const pg = svg.append('g')
		.data([this.lineData])
		.attr('class', 'pointer')
		.attr('transform', `translate(${this._radius()},${this._radius()})`);

	  const pointer = pg.append('path');
	  if(['angular','chevron'].includes(this.config.pointerTail)){
		pointer.attr('d', d3.line());
	  }else{
		pointer.attr('d',this.roundData);
	  }
	  pointer.attr('transform', `rotate(${this.config.minAngle})`);
	  
	  // display arc feet value
	  if((this.config.leftFootVal !== '') || (this.config.rightFootVal !== '')){
		  const FootDiv = d3.select(container).append('div')
			.style('width', `${(this.config.size - this.config.margin)}px`);

		  const leftFootVal = FootDiv.append('div')
			.attr('class', 'left-foot')
			.style('width', `${(this.config.size - this.config.margin)/2}px`)
			.text(d => this.config.leftFootVal)
			.style('color', this.config.lowThreshholdColor);

		  const rightFootVal = FootDiv.append('div')
			.attr('class', 'right-foot')
			.style('width', `${(this.config.size - this.config.margin)/2}px`)
			.text(d => this.config.rightFootVal)
			.style('color', this.config.highThreshholdColor);
	  }
	  
	  // display current value
	  const numberDiv = d3.select(container).append('div')
		.attr('class', 'number-div')
		.style('width', `${this.config.size - this.config.margin}px`);

	  const numberUnit = numberDiv.append('span')
		.attr('class', 'number-unit')
		.text(d => this.config.displayUnit);

	  const numberValue = numberDiv.append('span')
		.data([newValue])
		.attr('class', 'number-value')
		.text(d => d === undefined ? 0: d);
			
	  this.pointer = pointer;
	  this.numberValue = numberValue;

	}

/** Updating values for Pointer */
  update(newValue) {

	var newAngle = this.config.minAngle;
	if(this.config.maxValue < newValue){
		newAngle = this.config.maxAngle;
	}else
	if(this.config.minValue < newValue){
		newAngle = this.config.minAngle + (this.scale(newValue) * this.angleRange);
	}
	
	const round = this.config.roundMes;
	const unit_pre = this.config.unitMesPre;
	const unit_suf = this.config.unitMesSuf;
	const locale_mes = this.config.localeMes;
	
    this.pointer.transition()
      .duration(this.config.transitionMs)
      .attr('transform', `rotate(${newAngle})`);

	if(this.config.gradientColors){
		this.numberValue
		  .data([newValue])
		  .transition()
		  .duration(this.config.transitionMs)
		  .style('color', this.colorScale2(newAngle*Math.PI/180)) // NEW	  
		  .tween("", function(d) {
			const interpolator = d3.interpolate( this.textContent, d );
			const that = this;
			return function( t ) {
			  that.textContent = unit_pre+interpolator(t).toLocaleString(locale_mes,{minimumFractionDigits: round, maximumFractionDigits: round})+unit_suf;
			};
		  });
	} else {
		this.numberValue
		  .data([newValue])
		  .transition()
		  .duration(this.config.transitionMs)
		  .style('color', this.colorScale( this.scale(newValue) ))
		  .tween("", function(d) {
			const interpolator = d3.interpolate( this.textContent, d );
			const that = this;
			return function( t ) {
			  that.textContent = unit_pre+interpolator(t).toLocaleString(locale_mes,{minimumFractionDigits: round, maximumFractionDigits: round})+unit_suf;
			};
		  });
	}
  }

}