// ==============================================//
// Fichier					: n4v_hc_tools.js
// Version 					: 1.0
// Dernière modification 	: 11/10/2022
// ==============================================//

/**
 * archimedeanSpiral - Gives a set of cordinates for an Archimedian Spiral. 
 * Used in wordcloud.
 *
 * @param {number} t How far along the spiral we have traversed.
 * @return {object} Resulting coordinates, x and y.
 */
var archimedeanSpiral = function archimedeanSpiral(t) {
    t *= 0.1;
    return {
        x: t * Math.cos(t),
        y: t * Math.sin(t)
    };
};

/**
 * squareSpiral - Gives a set of coordinates for an rectangular spiral.
 * Used in wordcloud.
 *
 * @param {number} attempt How far along the spiral we have traversed.
 * @param {object} params Additional parameters.
 * @return {boolean|object} Resulting coordinates, x and y. False if the word
 * should be dropped from the visualization.
 */
var squareSpiral = function squareSpiral(attempt) {
	var k = Math.ceil((Math.sqrt(attempt) - 1) / 2),
	t = 2 * k + 1,
	m = Math.pow(t, 2),
	isBoolean = function(x) {
		return typeof x === 'boolean';
	},
	result = false;
	t -= 1;
	if (attempt <= 10000) {
		if (isBoolean(result) && attempt >= m - t) {
			result = {
				x: k - (m - attempt),
				y: -k
			};
		}
		m -= t;
		if (isBoolean(result) && attempt >= m - t) {
			result = {
				x: -k,
				y: -k + (m - attempt)
			};
		}

		m -= t;
		if (isBoolean(result)) {
			if (attempt >= m - t) {
				result = {
					x: -k + (m - attempt),
					y: k
				};
			} else {
				result = {
					x: k,
					y: k - (m - attempt - t)
				};
			}
		}
		result.x *= 15
		result.y *= 15;
	}
	return result;
};

/**
 * getRandomPosition
 * Used in wordcloud.
 *
 * @param  {number} size
 * @return {number}
 */
var getRandomPosition = function getRandomPosition(size) {
	return Math.round((size * (Math.random() + 0.5)) / 2);
};

/**
 * getRotation - Calculates a number of degrees to rotate, based upon a number
 *     of orientations within a range from-to.
 * Used in wordcloud.
 *
 * @param  {type} orientations Number of orientations.
 * @param  {type} from The smallest degree of rotation.
 * @param  {type} to The largest degree of rotation.
 * @return {type} Returns the resulting rotation for the word.
 */
var getRotation = function getRotation(orientations, from, to) {
	var range = to - from,intervals,orientation;
	if(orientations > 1)
		intervals = range / (orientations - 1)
	else
		intervals = 0
	
	orientation = Math.floor(Math.random() * orientations);
	return from + (orientation * intervals);
};

/**
 * randomPlacement : deciding rotation and initial position of a word in the plotchart
 * Used in wordcloud.
 *
 * @param  {Point} point Point which the word is connected to.
 * @param {object} params Additional parameters.
 * @return {object} Resulting coordinates, x and y, and rotation of the word.
 */
var randomPlacement = function randomPlacement(point, options) {
  var field = options.field,
    r = options.rotation;
  return {
    x: getRandomPosition(field.width) - (field.width / 2),
    y: getRandomPosition(field.height) - (field.height / 2),
   rotation: getRotation(r.orientations, r.from, r.to)
  };
};