function N4VFormatter (value) {
    this.originalValue_ = value;
    this.stringValue_ = '';
    this.abbreviation_= '';
    this.decimalSeparator_= '';
    this.groupingSeparator_= '';
    this.init();
}

N4VFormatter.NO_NOTATION = "";
N4VFormatter.NO_SCALE = 0;
N4VFormatter.DISPLAYED_DIGITS = 4;
N4VFormatter.GROUP_DIGITS_COUNT = 3;
N4VFormatter.GROUPING_PRECISION = Math.pow(10, -(N4VFormatter.GROUP_DIGITS_COUNT + 1));
N4VFormatter.GROUPING_MULTIPLIER = Math.pow(10, N4VFormatter.GROUP_DIGITS_COUNT);
N4VFormatter.NOTATIONS = ["y", "z", "a", "f", "p", "n", "Âµ", "m", N4VFormatter.NO_NOTATION, "K", "M", "B", "T", "P", "E", "Z", "Y"];
N4VFormatter.SCALES    = [-21, -18, -15, -12, -9, -6, -3, N4VFormatter.NO_SCALE, 3, 6, 9, 12, 15, 18, 21, 24, 27];

N4VFormatter.prototype.init = function () {
    this.decimalSeparator_ = ".";
    this.groupingSeparator_ = ",";
};

N4VFormatter.prototype.getGroupingSeparator = function () {
    return this.groupingSeparator_;
};

N4VFormatter.prototype.getDecimalSeparator = function () {
    return this.decimalSeparator_;
};

N4VFormatter.prototype.setOriginalValue = function (value) {
    this.originalValue_ = value;
};

N4VFormatter.prototype.getN4VFormattedValue = function () {
    return this.stringValue_;
};

N4VFormatter.prototype.getN4VAbbreviationValue = function () {
    return this.abbreviation_;
};

N4VFormatter.prototype.format = function () {
    this.formatValue(this.originalValue_);
};

N4VFormatter.prototype.formatValue = function (value) {
    if (value == 0) {
        this.stringValue_ = "0";
        this.abbreviation_ = "";
        return;
    }

    var absValue = Math.abs(value);
    var n = Math.log(absValue) / Math.log(10);
    var scale = Math.round(n);
    var token = N4VFormatter.NOTATIONS[N4VFormatter.NOTATIONS.length - 1];
    var previousToken = N4VFormatter.NOTATIONS[0];
    var appliedScale = N4VFormatter.GROUP_DIGITS_COUNT - N4VFormatter.SCALES[N4VFormatter.SCALES.length - 1];

    var newValue;
    var notationsCount = N4VFormatter.NOTATIONS.length;
    for (var notationIndex = 0; notationIndex < notationsCount; notationIndex++) {
        var currentScale = N4VFormatter.SCALES[notationIndex];
        if (scale < currentScale) {
            token = N4VFormatter.NOTATIONS[notationIndex];
            appliedScale = N4VFormatter.GROUP_DIGITS_COUNT - currentScale;
            if (notationIndex > 0) {
                previousToken = N4VFormatter.NOTATIONS[notationIndex - 1];
            }

            break;
        }
    }

    // special case for 0.1 & 0.01 & 0.001 like numbers
    if (scale < 0 && scale > -4)
    {
        token = N4VFormatter.NO_NOTATION;
        appliedScale = N4VFormatter.NO_SCALE;
    }

    newValue = value * Math.pow(10, appliedScale);

    var hasNoDecimal = (newValue == Math.floor(newValue));
    var isScalePositive = scale >= 0;
    var scaledValueString = '';
    var significantDigitsCount = 0;
    if (hasNoDecimal)
    {
        scaledValueString = this.formatIntegerValue( newValue);
        if (token !== '')
        {
            significantDigitsCount = scaledValueString.length;
            if (newValue < 0)
            {
                significantDigitsCount--;
            }
            if (significantDigitsCount < 2)
            {
                newValue *= N4VFormatter.GROUPING_MULTIPLIER;
                var tempValue = newValue;
                var roundedTempValue = Math.round(tempValue);
                if (Math.abs(tempValue - roundedTempValue) < N4VFormatter.GROUPING_PRECISION)
                {
                    newValue = roundedTempValue;
                }

                hasNoDecimal = newValue == Math.floor(newValue);
                if (hasNoDecimal)
                {
                    scaledValueString = this.formatIntegerValue( newValue);
                }
                else
                {
                    scaledValueString = this.formatDecimalValue(newValue);
                }
                token = previousToken;
            }
            else
            {
                scaledValueString = this.formatIntegerValue(newValue);
            }
        }
    }
    else
    {
        var scaledValueString = newValue.toString();
        if (token !=='')
        {
            var significantDigitsCount = isScalePositive ? scaledValueString.indexOf("."): scaledValueString.length - 1;
            if (newValue < 0)
            {
                significantDigitsCount--;
            }

            if (significantDigitsCount < 2)
            {
                newValue *= N4VFormatter.GROUPING_MULTIPLIER;
                var  tempValue = newValue;
                var  roundedTempValue = Math.round(tempValue);
                if (Math.abs(tempValue - roundedTempValue) < N4VFormatter.GROUPING_PRECISION)
                {
                    newValue =  roundedTempValue;
                }

                hasNoDecimal = newValue == Math.floor(newValue);
                if (hasNoDecimal)
                {
                    scaledValueString = this.formatIntegerValue( newValue);
                }
                else
                {
                    scaledValueString = this.formatDecimalValue(newValue);
                }
                token = previousToken;
            }
            else
            {
                newValue = this.getRoundedValue(newValue);
                scaledValueString = this.formatDecimalValue(newValue);
            }
        }
        else
        {
            newValue = this.getRoundedValue(newValue);
            scaledValueString = this.formatDecimalValue(newValue);
        }
    }

    this.stringValue_ = scaledValueString;
    this.abbreviation_ = token;
}


N4VFormatter.prototype.getRoundedValue = function (value)
{
    var newValue = value * N4VFormatter.GROUPING_MULTIPLIER;
    var tempValue = newValue;
    var roundedTempValue = Math.round(tempValue);
    if (Math.abs(tempValue - roundedTempValue) < N4VFormatter.GROUPING_PRECISION) {
        newValue = roundedTempValue;
    }
    newValue /= N4VFormatter.GROUPING_MULTIPLIER;

    return newValue;
};


N4VFormatter.prototype.formatDecimalValue = function( value)
{
    var isNegative = value < 0;
    var startIndex = isNegative ? 1 : 0;

    var valueString = value.toString();
    var decimalSeparatorIndex = valueString.indexOf(".");
    var digitsCount = decimalSeparatorIndex - startIndex;

    var formattedValue= '';

    // integer part
    var IntegerPartString = valueString.substring(0, decimalSeparatorIndex);
    if (digitsCount > N4VFormatter.GROUP_DIGITS_COUNT)
    {
        var integerPart = IntegerPartString;
        integerPart = integerPart.split('').reverse().join('');
        formattedValue = integerPart.substring(0, N4VFormatter.GROUP_DIGITS_COUNT);
        formattedValue = formattedValue.concat(this.groupingSeparator_);
        formattedValue = formattedValue.concat(integerPart.substring(N4VFormatter.GROUP_DIGITS_COUNT, integerPart.length));
        formattedValue = formattedValue.split('').reverse().join('');
    }
    else
    {
        formattedValue = IntegerPartString;
    }

    // decimal part
    var decimalCount = N4VFormatter.DISPLAYED_DIGITS - digitsCount;
    if (decimalCount > 0)
    {
        var decimalStartIndex = decimalSeparatorIndex + 1;
        var valueStringBuilderLength = valueString.length;
        var decimalEndIndex = Math.min(valueStringBuilderLength, decimalStartIndex + decimalCount);
        var decimalPartString = valueString.substring(decimalStartIndex, decimalEndIndex);
        var result = decimalPartString;
        result = result.split('').reverse().join('');
        var decimalValue = result.toString();
        if (decimalValue != 0)
        {
            formattedValue =  formattedValue.concat(this.decimalSeparator_);
            result = decimalValue;
            result = result.split('').reverse().join('');
            formattedValue = formattedValue.concat(result);
        }
    }

    return formattedValue;
};

N4VFormatter.prototype.formatIntegerValue = function (value) {
    var isNegative = value < 0;
    var startIndex = isNegative ? 1 : 0;

    var stringValue = value.toString();
    var charsCount = stringValue.length;
    var digitsCount = charsCount - startIndex;

    var formattedValue;
    if (digitsCount > N4VFormatter.GROUP_DIGITS_COUNT) {
        stringValue = stringValue.split("").reverse().join("");
        formattedValue = stringValue.substring(0, N4VFormatter.GROUP_DIGITS_COUNT);
        formattedValue = formattedValue.concat(this.groupingSeparator_);
        formattedValue = formattedValue.concat(stringValue.substring(N4VFormatter.GROUP_DIGITS_COUNT, charsCount));
        formattedValue = formattedValue.split("").reverse().join("");
    }
    else {
        formattedValue = stringValue.substring(0, charsCount);
    }

    return formattedValue;
};

if(typeof module!== 'undefined') {
    module.exports = N4VFormatter;
}
