// ==============================================//
// Fichier					: n4v_tf_fonc.js
// Version 					: 1.4
// Dernière modification 	: 15/11/2023
// ==============================================//

function n4v_calc(tf, colOps, colIndex, operation, onlyVisible, labelId, nb_prec, unit_pre, unit_suf, thousand_sep, decim_sep, format) {
	var colValues = tf.getFilteredDataCol(colIndex, false, true, [], onlyVisible);
	var result = 0;
	if(operation === "sum") {
		result = colOps.calcSum(colValues);
	}
	else if(operation === "mean") {
		result = colOps.calcMean(colValues) || 0;
	}
	else if(operation === "median") {
		result = colOps.calcMedian(colValues) || 0;
	}
	else if(operation === "min") {
		let calc = colOps.calcMin(colValues);
		result = isFinite(calc) ? calc : 0;
	}
	else if(operation === "max") {
		let calc = colOps.calcMax(colValues);
		result = isFinite(calc) ? calc : 0;
	}
	switch (format) {
		case "np1":
			if(result>0)
				unit_pre = unit_pre+"+";
			break;
		case "np2":
			if(result<0){
				unit_pre = "("+unit_pre;
				unit_suf = unit_suf+")";
				result = result*(-1);
			}
			break;
		default:
			break;
	}	
	result = result.toLocaleString('fr-FR', {minimumFractionDigits: nb_prec, maximumFractionDigits: nb_prec});//123 456,79
	result = result.replace(',',decim_sep);
	result = result.replaceAll(' ',thousand_sep);
		
	document.getElementById(labelId).innerHTML = unit_pre+result+unit_suf;
}

function add_btn_ldiv(label){
	var ldiv = document.getElementsByClassName("ldiv");
	/*
	var expdiv = document.createElement("div");
	var xlsexp = document.createElement("BUTTON");
	xlsexp.innerHTML = "XLS";
	xlsexp.setAttribute("id","xls_exp");
	xlsexp.setAttribute("onclick","exportTableToExcel(document, 'main_table')");
	xlsexp.setAttribute("title","Export filtered data to xls file");
	expdiv.appendChild(xlsexp);
	ldiv[0].appendChild(expdiv);
	*/
	var valdiv = document.createElement("div");
	valdiv.setAttribute("id","valdiv");
	var valsel = document.createElement("BUTTON");
	valsel.innerHTML = label;
	valsel.setAttribute("id","val_sel");
	valsel.setAttribute("onclick","click_val()");
	valsel.setAttribute("title","Apply this selection on related blocks");
	valdiv.appendChild(valsel);
	ldiv[0].appendChild(valdiv);
}

function add_btn_val(label_val,img_val,label_clrval,img_clrval){
	var rdiv = document.getElementsByClassName("rdiv");
	var valdiv = document.createElement("span");	
	var valsel = document.createElement("a");
	valsel.innerHTML = img_val;
	valsel.setAttribute("class","valsel");
	valsel.setAttribute("href","javascript:void(0);");
	valsel.setAttribute("onclick","click_val()");
	valsel.setAttribute("title",label_val);
	valdiv.appendChild(valsel);
	var valunsel = document.createElement("a");
	valunsel.innerHTML = img_clrval;
	valunsel.setAttribute("class","valsel");
	valunsel.setAttribute("href","javascript:void(0);");
	valunsel.setAttribute("onclick","click_clrval()");
	valunsel.setAttribute("title",label_clrval);
	valdiv.appendChild(valunsel);
	rdiv[0].appendChild(valdiv);
}

function add_btn_quit(label){
	var rdiv = document.getElementsByClassName("rdiv");
	var valdiv = document.createElement("span");	
	var valsel = document.createElement("a");
	valsel.setAttribute("href","javascript:void(0);");
	valsel.setAttribute("class","clb");
	valsel.setAttribute("onclick","closewindow()");
	valsel.setAttribute("title",label);
	valdiv.appendChild(valsel);
	rdiv[0].appendChild(valdiv);
}

function add_btn_fs(label,img,container){
	var rdiv = document.getElementsByClassName("rdiv");
	var valdiv = document.createElement("span");	
	var valsel = document.createElement("a");
	valsel.innerHTML = img;
	valsel.setAttribute("href","javascript:void(0);");
	valsel.setAttribute("class","fsb");
	valsel.setAttribute("onclick","toggleFullScreen('"+container+"');");
	valsel.setAttribute("title",label);
	valdiv.appendChild(valsel);
	rdiv[0].appendChild(valdiv);
}

function exportTableToExcel(mydocument, tableID, filename = ''){
    var downloadLink;
    var dataType = 'application/vnd.ms-excel';
    var tableSelect = mydocument.getElementById(tableID);
	var table_dup = tableSelect.cloneNode(true);
		
	var captions = table_dup.getElementsByTagName("caption")	
	if(captions.length == 1)
	    captions[0].outerHTML = '';
    var filters = table_dup.getElementsByClassName("fltrow")
	if(filters.length == 1)	
		filters[0].outerHTML = '';
	var imgs = table_dup.getElementsByClassName("sort-arrow")
	var cpt_imgs = imgs.length-1;
	for(var i=cpt_imgs; i>=0; i--){
		imgs[i].outerHTML = '';	
	}
	var tr_filtered = table_dup.getElementsByTagName("tr")
	var cpt_tr = tr_filtered.length-1;
	for(var i=cpt_tr; i>=0; i--){
		if(tr_filtered[i].style.display == "none")
			tr_filtered[i].outerHTML = '';	
	}
	
    var tableHTML = table_dup.outerHTML.replace(/ /g, '%20');
 	
    filename = filename?filename+'.xls':'excel_data.xls';
    downloadLink = mydocument.createElement("a");
    mydocument.body.appendChild(downloadLink);
    
	if(parent != null){ 
		if(parent.document != null){ 
			var iframe = parent.document.activeElement; 
			iframe.sandbox.add('allow-downloads');
		} 
	}
	
    if(navigator.msSaveOrOpenBlob){
        var blob = new Blob(['\ufeff', tableHTML], {
            type: dataType
        });
        navigator.msSaveOrOpenBlob( blob, filename);
    }else{
        downloadLink.href = 'data:' + dataType + ', ' + tableHTML;
        downloadLink.download = filename;
        downloadLink.click();	
    }
}

function checkall(id,idx){
var tab_filt=[];
$('#'+id).find("input[data-idx!='0']:visible").prop("checked", true).each(function(index){
	tab_filt.push($(this).val());	
});
tf.setFilterValue(idx, tab_filt);
tf.filter();
}

function checknone(id,idx){
$('#'+id).find("input[data-idx!='0']:visible").prop("checked", false)
tf.setFilterValue(idx, '');
tf.filter();
}

function viewall(id){
$('#'+id).find("input[data-idx!='0']:not(:visible)").each(function() {
	return $(this).parent().parent().show();
});
}

function viewselect(id){
$('#'+id).find("input[data-idx!='0']").each(function() {
	if ($(this).is(':checked')) {
		return $(this).parent().parent().show();
	}else{
		return $(this).parent().parent().hide();
	}
});
}

function viewunselect(id){
$('#'+id).find("input[data-idx!='0']").each(function() {
	if ($(this).is(':checked')) {
		return $(this).parent().parent().hide();
	}else{
		return $(this).parent().parent().show();
	}
});
}

function searchsel(rech, id){
viewall(id);
var txt=rech.value.toLowerCase();
$('#'+id).find("input[data-idx!='0']").each(function() {
var idx=$(this).attr('value');
	if ($(this).attr('value').toLowerCase().indexOf(txt)<0) {
		return $(this).parent().parent().hide();
	}else{
		return $(this).parent().parent().show();
	}
});
}

function clearinput_flt(id){
	document.getElementById('filter_'+id).value = '';
	viewall(id);
}

function clearinput_txt(id){
	document.getElementById(id).value = '';
	tf.filter();
}

function close_flt(id){
$('#'+id).parent().hide();
}

function close_ipt(id){
$('#'+id).parent().parent().hide();
}